/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  
  
  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 
  
      http://www.apache.org/licenses/LICENSE-2.0 
  
  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/

#include "glite/ce/monitor-client-api-c/CEEvent.h"
#include "glite/ce/monitor-client-api-c/Dialect.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include "glite/ce/monitor-client-api-c/GenericException.h"
#include "glite/ce/monitor-client-api-c/TopicNotSupportedException.h"
#include "glite/ce/monitor-client-api-c/DialectNotSupportedException.h"
#include "glite/ce/monitor-client-api-c/ServiceNotFoundException.h"
#include "glite/ce/monitor-client-api-c/AuthenticationException.h"
#include "glite/ce/monitor-client-api-c/CEMON_CLIENT.nsmap"
#include <iostream>
#include <string>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

using namespace std;
//using namespace glite::ce::monitor_client_api::soap_proxy;
using namespace glite::ce::monitor_client_api::cemon_exceptions;
namespace cemon_api = glite::ce::monitor_client_api::soap_proxy;
Namespace namespaces[] = {};

void showCEError(CEException& ex) {
  cerr << "CEMon raised a fault:"<<endl;
  cerr << "MethodName  =[" << ex.getMethod()<<"]"<<endl;
  cerr << "Timestamp   =[" << ex.getTimeStamp()<<"]"<<endl;
  cerr << "ErrorCode   =[" << ex.getErrorCode()<<"]"<<endl;
  cerr << "Description =[" << ex.getDescription()<<"]"<<endl;
  cerr << "FaultCause  =[" << ex.getFaultCause()<<"]"<<endl;
}

void showError(AbsException& ex) {
  cerr << "Client exception: "<<endl;
  cerr << "Error=[" << ex.what()<<"]"<<endl;
  exit(1);
}

int main(int argc, char *argv[])
{

  if(argc < 6) {
    cout << "Usage: " << argv[0] << " <cert_proxy> <cert_path> <service_URL_address> <topic_name> <dialect_name>" << endl;
    exit(0);
  }

  cemon_api::CEEvent *ceE;

  cemon_api::Topic topic = cemon_api::Topic(string(argv[4]));
//  Topic topic(string(argv[4]));


  if(string(argv[5])!="") {
    cemon_api::DialectW* dial = new cemon_api::DialectW(string(argv[5]));
    topic.addDialect(dial);
  }

  try{
    ceE = new cemon_api::CEEvent(string(argv[1]), string(argv[2]));
  } catch(AuthenticationInitException& authNEx) {
    cerr << "Authentication init Error: ["<<authNEx.what() <<"]"<< endl;
    exit(1);
  } catch(GeneralException& ex) {
    cerr << "Error: ["<<ex.what() <<"]"<< endl;
    exit(1);
  }

  ceE->setServiceURL(string(argv[3]));
  ceE->setRequestParam(&topic);
  int j=0;
  int max;
  
  if(argv[6])
    max = atoi(argv[6]);
  else
    max = 10;
    
  char *t;
  
  list< vector<string> > event_Messages;
  
  try {
  
      ceE->getTopicEvents( event_Messages );
      //ceE->getEvent();
      
  } catch (CEException& ex) {
    showCEError(ex);
    exit(1);
  } catch(AbsException& abs) {
    showError(abs);
    exit(1);
  }
    
//   do {
//     
//     int k=0;
//     while((t=(char *)ceE->getNextEventMessage())!=NULL)
//       cout << "Message[" << k++<<"]="<<t<<endl;
//     
//   } while( ceE->nextEvent() );
  int i = 0;
  for(list<vector<string> >::const_iterator it = event_Messages.begin();
      it != event_Messages.end();
      ++it)
  {
    cout << "Event[" << i++ << "]:"<< endl;
    int j = 0;
    for(vector<string>::const_iterator mit = it->begin();
        mit != it->end();
	++mit)
	{
	  cout << "\tMessage["<<j++<<"]=["<<*mit<<"]"<<endl;
	 
	}
  }
   
  ceE->cleanup();
  delete(ceE); // call CEEvent's DTOR (doing nothing) and then the AbsRequest's DTOR (that calls soap_end(...))
}
