/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  
  
  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 
  
      http://www.apache.org/licenses/LICENSE-2.0 
  
  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/

#include "glite/ce/monitor-client-api-c/CESubscriptionMgr.h"
#include "glite/ce/monitor-client-api-c/AbsException.h"
#include "glite/ce/monitor-client-api-c/GenericException.h"
#include "glite/ce/monitor-client-api-c/ServiceNotFoundException.h"
#include "glite/ce/monitor-client-api-c/AuthenticationException.h"
#include "glite/ce/monitor-client-api-c/GeneralException.h"
#include "glite/ce/monitor-client-api-c/CEMON_CLIENT.nsmap"
#include <iostream>
#include <string>
#include <ctime>
#include <cstring>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <boost/scoped_ptr.hpp>

using namespace std;
using namespace glite::ce::monitor_client_api::soap_proxy;
using namespace glite::ce::monitor_client_api::cemon_exceptions;

Namespace namespaces[] = {};

void showCEError(CEException& ex) {
  cerr << "CEMon raised a fault:"<<endl;
  cerr << "MethodName =" << ex.getMethod()<<endl;
  cerr << "Timestamp  =" << ex.getTimeStamp()<<endl;
  cerr << "ErrorCode  =" << ex.getErrorCode()<<endl;
  cerr << "Description=" << ex.getDescription() << endl;
  cerr << "FaultCause =" << ex.getFaultCause() << endl;
}

void showError(AbsException& ex) {
  cerr << "Client exception: "<<endl;
  cerr << "Error=" << ex.what()<<endl;
}

int main(int argc, char *argv[])
{
  if(argc < 8) {
    cerr << "Usage: " << argv[0] << " <cert_proxy> <cert_path> <service_URL_address> <SubID_to_update> <newConsumer> <newTopic> <newDialect> <duration> <newRate>"<<endl;
    exit(0);
  }

  //while(true) {
    CESubscriptionMgr *ceS;
    
    try{
      ceS = new CESubscriptionMgr(argv[1], argv[2]);
    } catch(AuthenticationInitException& authNEx) {
      cerr << "Authentication Error: ["<<authNEx.what() <<"]"<< endl;
      exit(1);
    } catch(GeneralException& ex) {
      cerr << "Error: ["<<ex.what() <<"]"<< endl;
      exit(1);
    }
    
    //   cout << "New topic="<<argv[6]<<endl;
    //   cout << "New dial ="<<argv[7]<<endl;
    //   cout << "SubID    ="<<argv[4]<<endl;
    //   cout << "Consumer ="<<argv[5]<<endl;
    
    Topic T(argv[6]);
    Policy P(atoi(argv[9]));
    T.addDialect(NULL);
    time_t newExpiration = time(NULL)+(time_t)atoi(argv[8]);
    try {
      ceS->update(argv[3], argv[4], argv[5], T, P, newExpiration);
    } catch (CEException& ex) {
      showCEError(ex);
      delete(ceS);
      exit(1);
    } catch(AbsException& abs) {
      showError(abs);
      delete(ceS);
      exit(1);
    } 
    delete(ceS);
    //}
}
